package ru.littlevictor.radio;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.PowerManager;
import android.util.DisplayMetrics;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;

public class RadioDownloader extends Activity {

	private String downloadUrl = "";
    private AlertDialog mDialog;
    private LayoutParams layoutParams;
	//-----------------------------------------------------------------------------
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	    //-----------------------------------------------------------------------------
		Uri inputData = getIntent().getData();
	    String fileName = "";
    	String scheme = null;
		if(inputData == null) {
			Toast.makeText(this, getText(R.string.invalid_url), Toast.LENGTH_LONG).show();
			finish();
		} else {
		    try 
		    {
		    	scheme = inputData.getScheme();
		    }
		    catch (Exception e) 
		    {
				Toast.makeText(this, getText(R.string.invalid_url), Toast.LENGTH_LONG).show();
				finish();
		    }
		    if(scheme.equalsIgnoreCase("http")) {
				downloadUrl = inputData.toString();
				if(getIntent().hasExtra("internal")) {
		            fileName = inputData.getPath().substring(inputData.getPath().lastIndexOf("/") + 1);
		            String path = Storage.getPath(getApplicationContext(), Storage.TMP_DIR) + fileName;
		            File file = new File(path);
		            if(file.exists() && file.isFile() && file.canRead()) {
						finish();
						startService(new Intent(this, RadioService.class)
						.putExtra(getResources().getString(R.string.service_state), RadioService.LOAD)
						.putExtra(getResources().getString(R.string.data_path), path));
		            } else {
					    if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
					    	new MyTask(this).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, downloadUrl);
					    } else {
					    	new MyTask(this).execute(downloadUrl);
					    }
		            }
				} else {
					Storage.clearCache(this);
                    new MyTask(this).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, downloadUrl);
				}
			}
		}
	};
	
	// -----------------------------------------------------------------------------
	private class MyTask extends AsyncTask<String, Integer, String> {

	    private Context context;
	    private PowerManager.WakeLock mWakeLock;
	    private ProgressDialog mProgressDialog;
	    private String fileName = "";

		// -----------------------------------------------------------------------------
	    public MyTask(Context context) {
	        this.context = context;
	    };
	    
		// -----------------------------------------------------------------------------
		@Override
	    protected String doInBackground(String... sUrl) {
	        InputStream input = null;
	        OutputStream output = null;
	        HttpURLConnection connection = null;
	        
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            
	        try {
	        	URL url = new URL(sUrl[0]);
	            fileName = url.getPath().substring(url.getPath().lastIndexOf("/") + 1);
	            connection = (HttpURLConnection) url.openConnection();
	            connection.setReadTimeout(3000);
	            connection.setConnectTimeout(3500);
	            connection.setRequestMethod("GET");
	            connection.setDoInput(true);
	            connection.connect();

	            if (connection.getResponseCode() != HttpURLConnection.HTTP_OK) {
	                return "Server returned HTTP " + connection.getResponseCode()
	                        + " " + connection.getResponseMessage();
	            }

	            int fileLength = connection.getContentLength();
	            input = connection.getInputStream();
	            output = new FileOutputStream(Storage.getPath(context, Storage.TMP_DIR) + fileName);

	            byte data[] = new byte[4096];
	            long total = 0;
	            int count;
	            while ((count = input.read(data)) != -1) {
	                total += count;
	                if (fileLength > 0) publishProgress((int) (total * 100 / fileLength));
	                output.write(data, 0, count);
	            }
                output.close();
                input.close();
                connection.disconnect();
	        } catch (Exception e) {
	            return e.toString();
	        } finally {
	            try {
	                if (output != null)
	                    output.close();
	                if (input != null)
	                    input.close();
	            } catch (IOException ignored) {}
	            if (connection != null) connection.disconnect();
	        }
	        return null;
	    };
		
		// -----------------------------------------------------------------------------
	    @Override
	    protected void onProgressUpdate(Integer... progress) {
	        super.onProgressUpdate(progress);
			mProgressDialog.setProgress(progress[0]);
	    };
		
		// -----------------------------------------------------------------------------
	    @Override
	    protected void onPreExecute() {
	        super.onPreExecute();
			mProgressDialog = new ProgressDialog(RadioDownloader.this);
			mProgressDialog.setMessage(downloadUrl);
			mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
			mProgressDialog.setCancelable(true);
			mProgressDialog.setMax(100);
			mProgressDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
				@Override
				public void onCancel(DialogInterface dialog) {
					MyTask.this.cancel(true);
				};
			});
	        PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
	        mWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, getClass().getName());
	        mWakeLock.acquire();
	        mProgressDialog.show();
	    };
	    
		// -----------------------------------------------------------------------------
	    @Override
	    protected void onPostExecute(String result) {
	        mWakeLock.release();
	        mProgressDialog.dismiss();
	        if (result != null){
	        	showDownloadErrorDialog(result);
	        } else {
				if(getIntent().hasExtra("internal")) {
					finish();
					startService(new Intent(context, RadioService.class)
					.putExtra(getResources().getString(R.string.service_state), RadioService.LOAD)
					.putExtra(getResources().getString(R.string.data_path), Storage.getPath(context, Storage.TMP_DIR) + fileName));
				} else {
					finish();
					startActivity(new Intent(context, RadioServiceActivity.class)
					.putExtra("internal", true)
					.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK)
					.setData(Uri.parse("file:" + Storage.getPath(context, Storage.TMP_DIR) + fileName)));
				}
	        }
	    };
	    
		// -----------------------------------------------------------------------------
	    @Override
	    protected void onCancelled() {
	    	super.onCancelled();
			finish();
	    };
	};   

	// -----------------------------------------------------------------------------
	public void showDownloadErrorDialog(String msg) {
		int[] theme = ThemeChooser.getAppTheme(this);
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_message, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        LinearLayout dialogContent = (LinearLayout) dialog.findViewById(R.id.dialog_content);
		TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_download_error_title));
        layoutParams = dialogTitle.getLayoutParams();
        layoutParams.width = getDialogMinWidth();
        dialogTitle.setLayoutParams(layoutParams);
		TextView dialogMessage = (TextView) dialog.findViewById(R.id.dialog_message);
		dialogMessage.setText(msg);
        Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mDialog != null) mDialog.dismiss();
                finish();
            };
        });
        if(theme[0] == R.color.color_theme_primary_1) {
            dialogContent.setBackgroundResource(R.color.color_background_list_dark);
            dialogMessage.setTextColor(getResources().getColor(R.color.color_text_primary_dark));
            btnOk.setBackgroundResource(R.drawable.background_activated_dark);
        } else {
            dialogContent.setBackgroundResource(R.color.color_background_list_light);
            dialogMessage.setTextColor(getResources().getColor(R.color.color_text_primary_light));
            btnOk.setBackgroundResource(R.drawable.background_activated_light);
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
		builder.setView(dialog)
            .setCancelable(true)
            .setOnCancelListener(new DialogInterface.OnCancelListener() {
                @Override
                public void onCancel(DialogInterface dialog) {
                    finish();
                }
            });
        mDialog = builder.create();
        mDialog.show();
	};

    // -----------------------------------------------------------------------------
    private int getDialogMinWidth() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height > width) {
            return width - 40;
        } else {
            return height - 40;
        }
    };
};
