package ru.littlevictor.radio;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.Toolbar;
import android.util.DisplayMetrics;
import android.util.SparseBooleanArray;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnLongClickListener;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EdgeEffect;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.lang.reflect.Field;
import java.util.ArrayList;

//import android.util.Log;

public class RadioEdit extends ActionBarActivity {
	// -----------------------------------------------------------------------------
    private final int DIALOG_NONE = 0;
    private final int DIALOG_ADD = 1;
    private final int DIALOG_EDIT = 2;
    private final int DIALOG_DELETE = 3;
    private final int DIALOG_CODE_PAGE = 4;
    private int dialogIsShow = DIALOG_NONE;
    private int editPosition = 0;
    private int currentPosition = -1;
    private int orientation;
    private int[] theme;
    private int themeBaseColor;
    private int themeAccentColor;
    private boolean pause = false;
    private final int REQUEST_OPEN = 1;
	public static Data data;
    private AlertDialog mDialog;
	private DynamicListView list;
//	private DynamicListAdapter adapter;
	private SharedPreferences sharedPreferences;
	private String currentName;
	private String currentUrl;
	private String path;
	private Menu menu;
	private MenuItem menuSelectAll;
	private MenuItem menuSelectNone;
	private MenuItem menuDelete;
	private MenuItem menuFavorites;
    private MenuItem menuCodePage;
	private LinearLayout menuBar;
	private ImageButton btnAdd;
	private ImageButton btnSelectAll;
	private ImageButton btnSelectNone;
	private ImageButton btnDelete;
	private ImageButton btnFavorites;
    private ImageButton btnCodePage;
	private EditText dialogName;
	private EditText dialogURL;
	private String browseStartPath;
    private Toolbar actionBar;
    private ActionBarFragment actionBarFragment;
    private FragmentTransaction transaction;
    private ViewGroup.LayoutParams layoutParams;
    private CodePage codePage;

	// -----------------------------------------------------------------------------
	@Override
	protected void onCreate(Bundle savedInstanceState) {
//        Log.d("myLog", "RadioEdit onCreate");
        // -----------------------------------------------------------------------------
        theme = ThemeChooser.getAppTheme(this);
        themeBaseColor = getResources().getColor(theme[0]);
        themeAccentColor = getResources().getColor(theme[1]);
        setTheme(theme[2]);
        super.onCreate(savedInstanceState);
        // -----------------------------------------------------------------------------
		setContentView(R.layout.radio_edit);
        createActionBar(getResources().getConfiguration().orientation);
        list = (DynamicListView) findViewById(R.id.list);
        menuBar = (LinearLayout) findViewById(R.id.menu_bar);
        if(theme[0] == R.color.color_theme_primary_1) {
            list.setBackgroundResource(R.color.color_background_list_dark);
            brandGlowEffect(this, getResources().getColor(android.R.color.white));
            menuBar.setBackgroundResource(R.drawable.background_action_bar_dark);
        } else {
            list.setBackgroundResource(R.color.color_background_list_light);
            brandGlowEffect(this, themeAccentColor);
            menuBar.setBackgroundColor(themeBaseColor);
        }
        if(getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT) {
            menuBar.setVisibility(View.VISIBLE);
        } else {
            menuBar.setVisibility(View.GONE);
        }
        sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
		list.setChoiceMode(DynamicListView.CHOICE_MODE_MULTIPLE);
		btnAdd = (ImageButton) findViewById(R.id.btn_add);
		btnSelectAll = (ImageButton) findViewById(R.id.btn_select_all);
		btnSelectNone = (ImageButton) findViewById(R.id.btn_select_none);
		btnDelete = (ImageButton) findViewById(R.id.btn_delete);
		btnFavorites = (ImageButton) findViewById(R.id.btn_favorites);
        btnCodePage = (ImageButton) findViewById(R.id.btn_code_page);
		Uri inputData = getIntent().getData();
		if (inputData != null) {
			currentPosition = getIntent().getIntExtra("current_position", -1);
			try {
				path = inputData.getPath();
			} catch (Exception e) {
				showToast(getResources().getString(R.string.invalid_file_path), Toast.LENGTH_LONG);
				finish();
			}
            init(path);
		} else {
			showToast(getResources().getString(R.string.invalid_file_path), Toast.LENGTH_LONG);
			finish();
		}
	};

	// -----------------------------------------------------------------------------
	private AdapterView.OnItemClickListener ListOnItemClick = new AdapterView.OnItemClickListener() {
		@Override
		public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
			renewOptionsMenu();
//            saveInstanceState();
		};
	};

	// -----------------------------------------------------------------------------
	private AdapterView.OnItemLongClickListener ListOnItemLongClick = new AdapterView.OnItemLongClickListener() {
		@Override
		public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
            editPosition = position;
			showEditDialog(position);
			return true;
		};
	};

	// -----------------------------------------------------------------------------
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
//        Log.d("myLog", "RadioEdit onCreateOptionsMenu");
		this.menu = menu;
        actionBar = actionBarFragment.getActionBar();
        if (currentPosition >= 0) {
            btnCodePage.setVisibility(View.GONE);
            actionBar.setNavigationIcon(R.drawable.abc_ic_ab_back_mtrl_am_alpha);
        }
		orientation = getResources().getConfiguration().orientation;
		if(orientation == Configuration.ORIENTATION_PORTRAIT) {
	    	portraitOptionsMenu();
		} else if(orientation == Configuration.ORIENTATION_LANDSCAPE) {
	    	landscapeOptionsMenu();
	    }
		renewOptionsMenu();
		return true;
	};

    // -----------------------------------------------------------------------------
    private void createActionBar(int orientation) {
//        Log.d("myLog", "RadioEdit createActionBar");
        actionBarFragment = new ActionBarFragment();
        actionBarFragment.setParentActivity(this);
        Bundle args = new Bundle();
        if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_LANDSCAPE);
        } else {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_PORTRAIT);
        }
        args.putString(actionBarFragment.TITLE, getResources().getString(R.string.app_edit_name));
        args.putString(actionBarFragment.SUBTITLE, " ");
        actionBarFragment.setArguments(args);
        transaction = getSupportFragmentManager().beginTransaction();
        transaction.replace(R.id.action_bar_container, actionBarFragment);
        transaction.commit();
    };

	// -----------------------------------------------------------------------------
	private void portraitOptionsMenu() {
	    menu.clear();    
	    getMenuInflater().inflate(R.menu.radio_edit_portrait, menu);
		menuSelectAll = menu.findItem(R.id.action_select_all);
		menuSelectNone = menu.findItem(R.id.action_select_none);
		menuDelete = menu.findItem(R.id.action_delete);
		menuFavorites = menu.findItem(R.id.action_add_to_main);
        menuCodePage = menu.findItem(R.id.action_code_page);
        if (currentPosition < 0) {
            menuCodePage.setVisible(true);
        } else {
            menuCodePage.setVisible(false);
        }
    	menuBar.setVisibility(View.VISIBLE);
	};

	// -----------------------------------------------------------------------------
	private void landscapeOptionsMenu() {
    	menuBar.setVisibility(View.GONE);
	    menu.clear();    
	    getMenuInflater().inflate(R.menu.radio_edit_landscape, menu);
		menuSelectAll = menu.findItem(R.id.action_select_all);
		menuSelectNone = menu.findItem(R.id.action_select_none);
		menuDelete = menu.findItem(R.id.action_delete);
		menuFavorites = menu.findItem(R.id.action_add_to_main);
        menuCodePage = menu.findItem(R.id.action_code_page);
        if (currentPosition < 0) {
            menuCodePage.setVisible(true);
        } else {
            menuCodePage.setVisible(false);
        }
	};
	
	// -----------------------------------------------------------------------------
	private void renewOptionsMenu() {
        if (data.getPath().equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))) {
            actionBar.setTitle(getText(R.string.app_edit_name));
            actionBar.setSubtitle(getText(R.string.main_list));
            menuFavorites.setVisible(false);
            btnFavorites.setVisibility(View.GONE);
        } else {
            File file = new File(data.getPath());
            actionBar.setTitle(getText(R.string.app_edit_name));
            actionBar.setSubtitle(file.getName());
            menuFavorites.setVisible(true);
            btnFavorites.setVisibility(View.VISIBLE);
        }
		int checkedCount = 0;
		int len = list.getCount();
		SparseBooleanArray checked = list.getCheckedItemPositions();
		for (int i = 0; i < len; i++) {
			if (checked.get(i)) {
				checkedCount++;
			}
		}
		if (checkedCount > 0) {
			if (checkedCount == len) {
				menuSelectAll.setEnabled(false);
				btnSelectAll.setClickable(false);
				btnSelectAll.setImageResource(R.drawable.ic_menu_select_all_disabled);
			} else {
				menuSelectAll.setEnabled(true);
				btnSelectAll.setClickable(true);
				btnSelectAll.setImageResource(R.drawable.ic_menu_select_all);
			}
			menuSelectNone.setEnabled(true);
			btnSelectNone.setClickable(true);
			btnSelectNone.setImageResource(R.drawable.ic_menu_select_none);
			menuDelete.setEnabled(true);
			btnDelete.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					showDeleteDialog();
				};
			});
			btnDelete.setClickable(true);
			btnDelete.setImageResource(R.drawable.ic_menu_delete);
			if (data.getPath().equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))) {
				menuFavorites.setEnabled(false);
				btnFavorites.setOnClickListener(null);
				btnFavorites.setClickable(false);
				btnFavorites.setImageResource(R.drawable.ic_menu_favorites_disabled);
			} else {
				menuFavorites.setEnabled(true);
				btnFavorites.setOnClickListener(new OnClickListener() {
					@Override
					public void onClick(View v) {
						addToMain();
					};
				});
				btnFavorites.setClickable(true);
				btnFavorites.setImageResource(R.drawable.ic_menu_favorites);
			}
		} else {
			menuSelectAll.setEnabled(true);
			btnSelectAll.setClickable(true);
			btnSelectAll.setImageResource(R.drawable.ic_menu_select_all);
			menuSelectNone.setEnabled(false);
			btnSelectNone.setClickable(false);
			btnSelectNone.setImageResource(R.drawable.ic_menu_select_none_disabled);
			menuDelete.setEnabled(false);
			btnDelete.setOnClickListener(null);
			btnDelete.setClickable(false);
			btnDelete.setImageResource(R.drawable.ic_menu_delete_disabled);
			menuFavorites.setEnabled(false);
			btnFavorites.setOnClickListener(null);
			btnFavorites.setClickable(false);
			btnFavorites.setImageResource(R.drawable.ic_menu_favorites_disabled);
		}
	};

	// -----------------------------------------------------------------------------
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
            data.writeList();
			if (currentPosition >= 0) {
                setCurrentPosition();
            } else {
                if(codePage != null)codePage.setDefault();
            }
			setResult(RESULT_OK, getIntent());
			finish();
			return true;
		case R.id.action_add:
			showAddNewDialog();
			return true;
		case R.id.action_delete:
			showDeleteDialog();
			return true;
		case R.id.action_select_all:
			selectAll();
			return true;
		case R.id.action_select_none:
			selectNone();
			return true;
		case R.id.action_add_to_main:
			addToMain();
			return true;
        case R.id.action_code_page:
            showChangeCodePageDialog();
            return true;
		}
		return super.onOptionsItemSelected(item);
	};

	// -----------------------------------------------------------------------------
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK:
            data.writeList();
            if (currentPosition >= 0) {
                dialogIsShow = DIALOG_NONE;
                setCurrentPosition();
            } else {
                if(codePage != null)codePage.setDefault();
            }
			setResult(RESULT_OK, getIntent());
			finish();
			return true;
		}
		return super.onKeyDown(keyCode, event);
	};

	// -----------------------------------------------------------------------------
	private void init(String filePath) {
		data = new Data(this, filePath);
		if (!data.errorMessage.equalsIgnoreCase("")) {
			showToast(data.errorMessage, Toast.LENGTH_LONG);

		}
		if (currentPosition >= 0) {
			if (currentName == null) currentName = data.names.get(currentPosition);
			if (currentUrl == null) currentUrl = data.urls.get(currentPosition);
		}
		list.setAdapter(new DynamicListAdapter(this, data.index));
		list.setOnItemClickListener(ListOnItemClick);
		list.setOnItemLongClickListener(ListOnItemLongClick);
		btnAdd.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				showAddNewDialog();
			};
		});
		btnAdd.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_add), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnSelectAll.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				selectAll();
			};
		});
		btnSelectAll.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_select_all), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnSelectNone.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				selectNone();
			};
		});
		btnSelectNone.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_select_none), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnDelete.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_delete), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnFavorites.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_add_to_main), Toast.LENGTH_SHORT);
				return true;
			};
		});
        btnCodePage.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                showChangeCodePageDialog();
            };
        });
        btnCodePage.setOnLongClickListener(new OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {
                showToast(getResources().getString(R.string.action_code_page), Toast.LENGTH_SHORT);
                return true;
            };
        });
	};

    // -----------------------------------------------------------------------------
    private void showChangeCodePageDialog() {
        codePage = new CodePage(this);
        int selectedPosition = codePage.getIndex();
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.action_code_page));
        ListView list = (ListView) dialog.findViewById(android.R.id.list);
        if(theme[0] == R.color.color_theme_primary_1) {
            list.setBackgroundResource(R.color.color_background_list_dark);
        } else {
            list.setBackgroundResource(R.color.color_background_list_light);
        }
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams = list.getLayoutParams();
            layoutParams.height = (int)(getScreenMinPixels()/100*60);
            list.setLayoutParams(layoutParams);
        } else {
            layoutParams = list.getLayoutParams();
            layoutParams.height = (int)(getScreenMaxPixels()/100*70);
            list.setLayoutParams(layoutParams);
        }
        list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
        list.setAdapter(new AdapterDialogChooseValue(this, codePage.getNames(), selectedPosition));
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                dialogIsShow = DIALOG_NONE;
                codePage.setIndex(position);
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                init(path);
            }
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
            }
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
        list.setSelection(selectedPosition);
        dialogIsShow = DIALOG_CODE_PAGE;
    };

	// -----------------------------------------------------------------------------
	private void showAddNewDialog() {
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_add, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        LinearLayout dialogContent = (LinearLayout) dialog.findViewById(R.id.dialog_content);
        if(theme[0] == R.color.color_theme_primary_1) {
            dialogContent.setBackgroundResource(R.color.color_background_list_dark);
        } else {
            dialogContent.setBackgroundResource(R.color.color_background_list_light);
        }
		TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_add_new_title));
        layoutParams = dialogTitle.getLayoutParams();
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams.width = (int)(getScreenMaxPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        } else {
            layoutParams.width = (int)(getScreenMinPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        }
        dialogName = (EditText) dialog.findViewById(R.id.dialog_name);
        dialogName.setText("");
		dialogURL = (EditText) dialog.findViewById(R.id.dialog_url);
        dialogURL.setText("");
        ImageButton btnBrowse = (ImageButton) dialog.findViewById(R.id.btn_browse);
		btnBrowse.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                startActivityForResult(new Intent(getBaseContext(), DialogFile.class)
                        .putExtra(DialogFile.START_PATH, browseStartPath)
                        .putExtra(DialogFile.CAN_SELECT_DIR, false)
                        .putExtra(DialogFile.SELECTION_MODE, DialogFile.MODE_SELECT)
                        .putExtra(DialogFile.FORMAT_FILTER, getResources().getStringArray(R.array.media_file_extensions_array)), REQUEST_OPEN);
            };
        });
		btnBrowse.setOnLongClickListener(new OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {
                showToast(getResources().getString(R.string.action_select), Toast.LENGTH_SHORT);
                return true;
            };
        });
        btnBrowse.setVisibility(View.VISIBLE);
        Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                data.names.add(dialogName.getText().toString());
                data.urls.add(dialogURL.getText().toString());
                data.writeList();
                if (!data.errorMessage.equalsIgnoreCase("")) showToast(data.errorMessage, Toast.LENGTH_LONG);
                init(data.getPath());
                list.setSelection(data.size() - 1);
            };
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
            };
        });
		AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
        dialogIsShow = DIALOG_ADD;
	};

	// -----------------------------------------------------------------------------
	protected void onActivityResult(final int requestCode, int resultCode, final Intent intent) {
		if (resultCode == ActionBarActivity.RESULT_OK) {
			switch(requestCode) {
			case REQUEST_OPEN:
				String path = intent.getData().getPath();
				File file = new File(path);
				browseStartPath = file.getParent().toString();
				String name = file.getName();
				name = name.substring(0, name.lastIndexOf("."));
				if (dialogName.getText().length() == 0) {
					dialogName.setText(name);
				}
				dialogURL.setText(intent.getData().toString());
				break;
			};
		};
	};
    
	// -----------------------------------------------------------------------------
	private void showEditDialog(final int position) {
        final String name = data.names.get(position);
        final String url = data.urls.get(position);
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_add, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        LinearLayout dialogContent = (LinearLayout) dialog.findViewById(R.id.dialog_content);
        if(theme[0] == R.color.color_theme_primary_1) {
            dialogContent.setBackgroundResource(R.color.color_background_list_dark);
        } else {
            dialogContent.setBackgroundResource(R.color.color_background_list_light);
        }
		TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_edit_title));
        layoutParams = dialogTitle.getLayoutParams();
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams.width = (int)(getScreenMaxPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        } else {
            layoutParams.width = (int)(getScreenMinPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        }
		final EditText dialogName = (EditText) dialog.findViewById(R.id.dialog_name);
        dialogName.setText(name);
		final EditText dialogURL = (EditText) dialog.findViewById(R.id.dialog_url);
		dialogURL.setText(url);
        Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if ((name.equals(dialogName.getText().toString()) && (url.equals(dialogURL.getText().toString())))) {
                    dialogIsShow = DIALOG_NONE;
                    if(mDialog != null) mDialog.dismiss();
                    mDialog = null;
                    renewOptionsMenu();
                } else {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                    data.names.set(position, dialogName.getText().toString());
                    data.urls.set(position, dialogURL.getText().toString());
                    // ------------------------------------------------------------
                    SparseBooleanArray checkedSparse = list.getCheckedItemPositions();
                    int len = list.getCount();
                    boolean[] checked = new boolean[len];
                    for (int i = 0; i < len; i++) {
                        if (checkedSparse.get(i)) {
                            checked[i] = true;
                        } else {
                            checked[i] = false;
                        }
                    }
                    // ------------------------------------------------------------
                    data.writeList();
                    if (!data.errorMessage.equalsIgnoreCase("")) showToast(data.errorMessage, Toast.LENGTH_LONG);
                    init(data.getPath());
                    for (int i = 0; i < checked.length; i++) {
                        if (checked[i] == true) {
                            list.setItemChecked(i, true);
                        }
                    }
                    list.setSelection(position);
                    renewOptionsMenu();
                }
            };
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                renewOptionsMenu();
            };
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
        dialogIsShow = DIALOG_EDIT;
	};

	// -----------------------------------------------------------------------------
	private void showDeleteDialog() {
		int checkedCount = 0;
		int len = list.getCount();
		SparseBooleanArray checked = list.getCheckedItemPositions();
		for (int i = 0; i < len; i++) {
			if (checked.get(i)) {
				checkedCount++;
			}
		}
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_alert, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        LinearLayout dialogContent = (LinearLayout) dialog.findViewById(R.id.dialog_content);
        if(theme[0] == R.color.color_theme_primary_1) {
            dialogContent.setBackgroundResource(R.color.color_background_list_dark);
        } else {
            dialogContent.setBackgroundResource(R.color.color_background_list_light);
        }
		TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_delete_title));
        layoutParams = dialogTitle.getLayoutParams();
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams.width = (int)(getScreenMaxPixels()/100*60);
            dialogTitle.setLayoutParams(layoutParams);
        } else {
            layoutParams.width = (int)(getScreenMinPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        }
		TextView dialogMessage = (TextView) dialog.findViewById(R.id.dialog_message);
		dialogMessage.setText(getText(R.string.dialog_delete_message) + " "	+ String.valueOf(checkedCount));
        Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                ArrayList<String> namesTmp = new ArrayList<String>();
                ArrayList<String> urlsTmp = new ArrayList<String>();
                int len = list.getCount();
                SparseBooleanArray checked = list.getCheckedItemPositions();
                for (int i = 0; i < len; i++) {
                    if (!checked.get(i)) {
                        namesTmp.add(data.names.get(i));
                        urlsTmp.add(data.urls.get(i));
                    }
                }
                if (namesTmp.size() > 0) {
                    data.names = new ArrayList<String>(namesTmp);
                    data.urls = new ArrayList<String>(urlsTmp);
                } else {
                    data.names.clear();
                    data.urls.clear();
                    data.names.add("");
                    data.urls.add("");
                }
                data.writeList();
                if (!data.errorMessage.equalsIgnoreCase("")) showToast(data.errorMessage, Toast.LENGTH_LONG);
                init(data.getPath());
            };
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
            };
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
        dialogIsShow = DIALOG_DELETE;
	};

	// -----------------------------------------------------------------------------
	private void setCurrentPosition() {
		int i;
		int position = 0;
		for (i = 0; i < list.getCount(); i++) {
			if (data.names.get(i).equalsIgnoreCase(currentName)) {
				if (data.urls.get(i).equalsIgnoreCase(currentUrl)) {
					position = i;
					break;
				}
			}
		}
		sharedPreferences.edit().putInt("current_position", position).apply();
	};

    // -----------------------------------------------------------------------------
    private void brandGlowEffect(Context context, int brandColor) {
        if (Build.VERSION.SDK_INT >= 21)
        {
            EdgeEffect edgeEffectTop = new EdgeEffect(this);
            edgeEffectTop.setColor(brandColor);

            EdgeEffect edgeEffectBottom = new EdgeEffect(this);
            edgeEffectBottom.setColor(brandColor);

            try {
                Field f1 = AbsListView.class.getDeclaredField("mEdgeGlowTop");
                f1.setAccessible(true);
                f1.set(list, edgeEffectTop);

                Field f2 = AbsListView.class.getDeclaredField("mEdgeGlowBottom");
                f2.setAccessible(true);
                f2.set(list, edgeEffectBottom);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        else {
            int glowDrawableId = context.getResources().getIdentifier("overscroll_glow", "drawable", "android");
            Drawable androidGlow = context.getResources().getDrawable(glowDrawableId);
            androidGlow.setColorFilter(brandColor, PorterDuff.Mode.SRC_IN);
            int edgeDrawableId = context.getResources().getIdentifier("overscroll_edge", "drawable", "android");
            Drawable androidEdge = context.getResources().getDrawable(edgeDrawableId);
            androidEdge.setColorFilter(brandColor, PorterDuff.Mode.SRC_IN);
        }
    };

    // -----------------------------------------------------------------------------
    private void showToast(String message, int length) {
        Toast toast = Toast.makeText(this, message, length);
        toast.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL, 0, 0);
        if(theme[0] == R.color.color_theme_primary_1) {
            toast.getView().setBackgroundResource(R.drawable.background_toast_dark);
        } else {
            toast.getView().setBackgroundResource(R.drawable.background_toast_light);
        }
        toast.show();
    };

	// -----------------------------------------------------------------------------
	private void selectAll() {
        int len = list.getCount();
		for (int i = 0; i < len; i++) {
			list.setItemChecked(i, true);
		}
		renewOptionsMenu();
	};

	// -----------------------------------------------------------------------------
	private void selectNone() {
		int len = list.getCount();
        list.clearChoices();
		for (int i = 0; i < len; i++) {
			list.setItemChecked(i, false);
		}
		renewOptionsMenu();
	};

	// -----------------------------------------------------------------------------
	private void addToMain() {
		int len = list.getCount();
		int n = 0;
		SparseBooleanArray checked = list.getCheckedItemPositions();
		Data mainList = new Data(this, Storage.getPath(this, Storage.MAIN_LIST));
		for (int i = 0; i < len; i++) {
			if (checked.get(i)) {
				n++;
				mainList.names.add(data.names.get(i));
				mainList.urls.add(data.urls.get(i));
			}
		}
		mainList.writeList();
		if (mainList.errorMessage.equalsIgnoreCase("")) {
			showToast(String.valueOf(n) + " " + getText(R.string.dialog_add_message), Toast.LENGTH_LONG);
		} else {
			showToast(mainList.errorMessage, Toast.LENGTH_LONG);
		}
	};

    // -----------------------------------------------------------------------------
    private int getScreenMaxPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height >= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    private int getScreenMinPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height <= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onSaveInstanceState(Bundle outState) {
//        Log.d("myLog", "RadioEdit onSaveInstanceState");
        super.onSaveInstanceState(outState);
        SparseBooleanArray checkedSparse = list.getCheckedItemPositions();
        int len = list.getCount();
        boolean[] checked = new boolean[len];
        for (int i = 0; i < len; i++) {
            if (checkedSparse.get(i)) {
                checked[i] = true;
            } else {
                checked[i] = false;
            }
        }
        outState.putBooleanArray("checked", checked);
        outState.putInt("dialogIsShow", dialogIsShow);
        outState.putInt("editPosition", editPosition);
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
//        Log.d("myLog", "RadioEdit onRestoreInstanceState");
        super.onRestoreInstanceState(savedInstanceState);
        if(savedInstanceState != null) {
            boolean[] checked = savedInstanceState.getBooleanArray("checked");
            if(checked != null) {
                for (int i = 0; i < checked.length; i++) {
                    if (checked[i] == true) {
                        list.setItemChecked(i, true);
                    }
                }
            }
            switch(savedInstanceState.getInt("dialogIsShow", DIALOG_NONE)) {
                case DIALOG_ADD:
                    showAddNewDialog();
                    break;
                case DIALOG_EDIT:
                    showEditDialog(savedInstanceState.getInt("editPosition", editPosition));
                    break;
                case DIALOG_DELETE:
                    showDeleteDialog();
                    break;
                case DIALOG_CODE_PAGE:
                    showChangeCodePageDialog();
                    break;
            }
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onDestroy() {
//        Log.d("myLog", "RadioEdit onDestroy");
        if(mDialog != null) {
            mDialog.dismiss();
        }
        dialogIsShow = DIALOG_NONE;
        super.onDestroy();
    };

};
