package ru.littlevictor.radio;

//-----------------------------------------------------------------------------
import android.app.PendingIntent;
import android.app.Service;
import android.os.IBinder;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.widget.RemoteViews;
import ru.littlevictor.radio.WidgetMain.WidgetNextService;
import ru.littlevictor.radio.WidgetMain.WidgetPlayStopService;
import ru.littlevictor.radio.WidgetMain.WidgetPreviousService;

//-----------------------------------------------------------------------------
public class WidgetBtn extends AppWidgetProvider {

	private static int serviceState;
	private static RemoteViews widgetViews;

	//-----------------------------------------------------------------------------
	@Override
	public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
		
		super.onUpdate(context, appWidgetManager, appWidgetIds);
		Intent intent = new Intent(context, RadioService.class)
		.putExtra(context.getResources().getString(R.string.service_state), RadioService.STATE);
		context.startService(intent);
    };

	//-----------------------------------------------------------------------------
	private void update(Context context) {
		
		widgetViews = new RemoteViews(context.getPackageName(), R.layout.widget_btn);

		widgetViews.setOnClickPendingIntent(R.id.widgetBigImage, PendingIntent.getActivity(context, 0,
				new Intent(context, RadioMain.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
		
		widgetViews.setOnClickPendingIntent(R.id.widgetButtonPlayStop, PendingIntent.getService(context, 0,
				new Intent(context, WidgetBtnPlayStopService.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
		
		widgetViews.setOnClickPendingIntent(R.id.widgetButtonPrev, PendingIntent.getService(context, 0,
				new Intent(context, WidgetBtnPreviousService.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
		
		widgetViews.setOnClickPendingIntent(R.id.widgetButtonNext, PendingIntent.getService(context, 0,
				new Intent(context, WidgetBtnNextService.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
 		
        AppWidgetManager.getInstance(context).updateAppWidget(new ComponentName(context, WidgetBtn.class.getName()), widgetViews);
        
		updateView(context);
	};
		
	//-----------------------------------------------------------------------------
	private static void updateView(Context context) {
		
		switch (serviceState) {
			case RadioService.TERM:
			case RadioService.STOP:
				
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_launcher);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.shortcut_btn_play);
				break;
				
			case RadioService.ERROR:
				
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_notification_alert);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.shortcut_btn_stop);
				break;
				
			case RadioService.PLAY:
				
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_launcher);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.shortcut_btn_stop);
				break;
				
			case RadioService.LOAD:
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_notification_sync);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.shortcut_btn_stop);
				break;
		};
		
        AppWidgetManager.getInstance(context).updateAppWidget(new ComponentName(context, WidgetBtn.class.getName()), widgetViews);
	};
		
	//-----------------------------------------------------------------------------
    public static class WidgetBtnUpdateService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            int[] appWidgetIds = AppWidgetManager.getInstance(this).getAppWidgetIds(new ComponentName(this, WidgetBtn.class));
            if(appWidgetIds.length > 0) {
                serviceState = intent.getIntExtra(getResources().getString(R.string.service_state), RadioService.TERM);
                if(widgetViews == null) {
                    new WidgetBtn().update(getBaseContext());
                } else {
                    updateView(getBaseContext());
                }
            }
            stopSelf(startId);
            return startId;
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
    public static class WidgetBtnPlayStopService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            startService(new Intent(this, RadioService.class)
                    .putExtra("internal", true)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.PLAY));
            stopSelf(startId);
            return startId;
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
    public static class WidgetBtnPreviousService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            startService(new Intent(this, RadioService.class)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.PREV));
            stopSelf(startId);
            return startId;
        };

        // -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
    public static class WidgetBtnNextService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            startService(new Intent(this, RadioService.class)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.NEXT));
            stopSelf(startId);
            return startId;
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
};
