package ru.littlevictor.radio;

//-----------------------------------------------------------------------------
import android.app.PendingIntent;
import android.app.Service;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.widget.RemoteViews;

import java.util.ArrayList;

//-----------------------------------------------------------------------------
public class WidgetMain extends AppWidgetProvider {

	public static ArrayList<String> names;
	private static int serviceState;
	public static int currentPosition;
	private static String trackName = "";
	private static RemoteViews widgetViews;
	public static String theme;

	//-----------------------------------------------------------------------------
	@Override
	public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
		
		super.onUpdate(context, appWidgetManager, appWidgetIds);
		Intent intent = new Intent(context, RadioService.class)
		.putExtra(context.getResources().getString(R.string.service_state), RadioService.STATE);
		context.startService(intent);
    };

	//-----------------------------------------------------------------------------
	private void update(Context context, int[] appWidgetIds) {
		
		theme = PreferenceManager.getDefaultSharedPreferences(context).getString("widget_theme", "0");
		
		if (theme.equalsIgnoreCase("1")) {
			
			widgetViews = new RemoteViews(context.getPackageName(), R.layout.widget_main_dark);
			
		} else {
			
			widgetViews = new RemoteViews(context.getPackageName(), R.layout.widget_main_light);
		};

		widgetViews.setOnClickPendingIntent(R.id.widgetBigImage, PendingIntent.getActivity(context, 0,
				new Intent(context, RadioMain.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
		
		widgetViews.setOnClickPendingIntent(R.id.widgetButtonPlayStop, PendingIntent.getService(context, 0,
				new Intent(context, WidgetPlayStopService.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
		
		widgetViews.setOnClickPendingIntent(R.id.widgetButtonPrev, PendingIntent.getService(context, 0,
				new Intent(context, WidgetPreviousService.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
		
		widgetViews.setOnClickPendingIntent(R.id.widgetButtonNext, PendingIntent.getService(context, 0,
				new Intent(context, WidgetNextService.class)
				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
				PendingIntent.FLAG_CANCEL_CURRENT));
 		
		widgetViews.setRemoteAdapter(R.id.list, new Intent(context, WidgetSevice.class));
		
        AppWidgetManager.getInstance(context).updateAppWidget(new ComponentName(context, WidgetMain.class.getName()), widgetViews);
        
		updateView(context, appWidgetIds);
	};
		
	//-----------------------------------------------------------------------------
	private static void updateView(Context context, int[] appWidgetIds) {
		
		if(!theme.equalsIgnoreCase(PreferenceManager.getDefaultSharedPreferences(context).getString("widget_theme", "0"))) {
			
			new WidgetMain().update(context, appWidgetIds);
			return;
		};
		
		switch (serviceState) {
			case RadioService.TERM:
			case RadioService.STOP:
				
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_notification);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.ic_media_play);
				widgetViews.setTextViewText(R.id.widgetTrack, context.getResources().getString(R.string.no_name));
				break;
				
			case RadioService.ERROR:
				
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_notification_alert);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.ic_media_pause);
				widgetViews.setTextViewText(R.id.widgetTrack, context.getResources().getString(R.string.no_name));
				break;
				
			case RadioService.PLAY:
				
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_notification);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.ic_media_pause);
				widgetViews.setTextViewText(R.id.widgetTrack, trackName);
				break;
				
			case RadioService.LOAD:
				widgetViews.setImageViewResource(R.id.widgetBigImage, R.drawable.ic_notification_sync);
				widgetViews.setImageViewResource(R.id.widgetButtonPlayStop, R.drawable.ic_media_pause);
				widgetViews.setTextViewText(R.id.widgetTrack, context.getResources().getString(R.string.no_name));
				break;
		};
		
		widgetViews.setTextViewText(R.id.widgetTitle, names.get(currentPosition));
		
    	AppWidgetManager.getInstance(context).notifyAppWidgetViewDataChanged(appWidgetIds, R.id.list);
 		
 		widgetViews.setPendingIntentTemplate(R.id.list, PendingIntent.getService(context, 0,
 				new Intent(context, WidgetItemPositionService.class)
 				.setAction(AppWidgetManager.ACTION_APPWIDGET_CONFIGURE),
 				PendingIntent.FLAG_CANCEL_CURRENT));
    	widgetViews.setScrollPosition(R.id.list, currentPosition);
 		
        AppWidgetManager.getInstance(context).updateAppWidget(new ComponentName(context, WidgetMain.class.getName()), widgetViews);
	};
		
	//-----------------------------------------------------------------------------
    public static class WidgetUpdateService extends Service {

    	// -----------------------------------------------------------------------------
        public void onStart(Intent intent, int startId) {
        	
            int[] appWidgetIds = AppWidgetManager.getInstance(this).getAppWidgetIds(new ComponentName(this, WidgetMain.class));
            
            if(appWidgetIds.length > 0) {
            	
    			names = intent.getStringArrayListExtra(getResources().getString(R.string.media_name));
    			
    			if(names == null || names.size() < 1) {
    	            stopSelf(startId);
    				return;
    			};
    			
				currentPosition = intent.getIntExtra(getResources().getString(R.string.media_position), -1);
				
    			if(currentPosition < 0 || currentPosition >= names.size()) {
    	            stopSelf(startId);
    				return;
    			};
    			
            	trackName = intent.getStringExtra(getResources().getString(R.string.media_track));
    			serviceState = intent.getIntExtra(getResources().getString(R.string.service_state), RadioService.TERM);
    			
                if(widgetViews == null) {

                	new WidgetMain().update(getBaseContext(), appWidgetIds);
                	
                } else {

                	updateView(getBaseContext(), appWidgetIds);
                	
                };
            };
            
            stopSelf(startId);
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
    public static class WidgetItemPositionService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            int position = intent.getIntExtra(getResources().getString(R.string.widget_item_position), 0);
            startService(new Intent(this, RadioService.class)
                    .putExtra("internal", true)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.POS)
                    .putExtra(getResources().getString(R.string.media_position), position));
            stopSelf(startId);
            return startId;
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
    public static class WidgetPlayStopService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            startService(new Intent(this, RadioService.class)
                    .putExtra("internal", true)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.PLAY));
            stopSelf(startId);
            return startId;
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
    public static class WidgetPreviousService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            startService(new Intent(this, RadioService.class)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.PREV));
            stopSelf(startId);
            return startId;
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
    public static class WidgetNextService extends Service {
        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            startService(new Intent(this, RadioService.class)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.NEXT));
            stopSelf(startId);
            return startId;
        };

    	// -----------------------------------------------------------------------------
        public IBinder onBind(Intent intent) {
            return null;
        };
    };
    
	//-----------------------------------------------------------------------------
};
